//************************
//Coded by Zhihao Wu
//Last update: June, 2012
//************************

#include <stdlib.h>
#include <stdio.h>
#include <iostream>
#include <string>

#include "graph_unweighted.h"
#include "bmlpa_unweighted.h"


using namespace std;

char *inputfile = NULL;
char *n2c_outputfile = NULL;
char *c2n_outputfile =NULL;
double para_p = 0.7;


void parse_args(int argc, char **argv)
{
	int i;
	if(argc < 2)
	{
		cerr<<"options: "<<endl;
		cerr<<"giving inputfile using: -i inputfilepath"<<endl;
		cerr<<"giving outputfile in n2c format using: -on2c outputfilepath"<<endl;
		cerr<<"giving outputfile in c2n format using: -oc2n outputfilepath"<<endl;
		cerr<<"giving parameter p using: -p paramerter_p"<<endl;
		exit(0);
	}
	for(i = 1; i<argc; i++)
	{
		string temp = argv[i];
		if(temp == "-i")
		{
			inputfile = argv[i+1];
			i++;
		}else if(temp == "-p")
		{
			para_p = atof(argv[i+1]);
			i++;
		}else if(temp == "-on2c")
		{
			n2c_outputfile = argv[i+1];
			i++;
		}else if(temp == "-oc2n")
		{
			c2n_outputfile = argv[i+1];
			i++;
		}else{
			cerr<<"error parameter!"<<endl;
			cerr<<"options: "<<endl;
			cerr<<"giving inputfile using: -i inputfilepath"<<endl;
			cerr<<"giving outputfile in n2c format using: -on2c outputfilepath"<<endl;
			cerr<<"giving outputfile in c2n format using: -oc2n outputfilepath"<<endl;
			cerr<<"giving parameter p using: -p paramerter_p"<<endl;
			exit(0);
			
		}
	}
}

int main(int argc, char **argv)
{
	//parse parameters
	parse_args(argc, argv);

	//read the network data
	Graph *g;
	g = new Graph(inputfile, false);

	cout<<"number of nodes: "<<g->links.size()<<endl;
	cout<<"number of edges: "<<g->nb_links<<endl;

	Algorithm *algorithm;
	algorithm = new Algorithm(g);

	//set parameter p
	algorithm->setp(para_p);
	//execute bmlpa algorithm
	algorithm->execute_bmlpa();

	//output overlapping communities
	if(n2c_outputfile!=NULL)
	{
		algorithm->saveResult(n2c_outputfile, "n2c");
	}

	if(c2n_outputfile!=NULL)
	{
		algorithm->saveResult(c2n_outputfile,"c2n");
	}
	cout<<"Done."<<endl;
	return 0;
}